@@ -2,11 +2,11 @@
 #
 
 .spdepOptions <- new.env(FALSE, globalenv())
-assign("spChkID", FALSE, env = .spdepOptions)
-assign("zeroPolicy", FALSE, env = .spdepOptions)
-assign("verbose", FALSE, env = .spdepOptions)
-assign("cl", NULL, env = .spdepOptions)
-assign("rlecuyerSeed", rep(12345, 6), env = .spdepOptions)
+assign("spChkID", FALSE, envir = .spdepOptions)
+assign("zeroPolicy", FALSE, envir = .spdepOptions)
+assign("verbose", FALSE, envir = .spdepOptions)
+assign("cl", NULL, envir = .spdepOptions)
+assign("rlecuyerSeed", rep(12345, 6), envir = .spdepOptions)
 
 #.conflicts.OK <- TRUE
 
